import java.lang.*;
import java.util.*;
import java.net.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

public class Air extends Canvas implements ActionListener {
  // parameters
  public final int WIDTH;
  public final int HEIGHT;
  public static final int MAV_RAD = 3;
  public static final Color MAV_COLOR = Color.black;
  public final int DIST;
  public final int G;
  public final double Fmax;
  public final double FR;

  boolean repulse;

  Vector MAVlist;  // the list of MAVs
  Console main;

  MouseAdapter blastListener;
  Cursor defaultCursor;
  Cursor bombCursor;

  // test test !!!!!
  double Vx, Vy;
  int vcount;

  public Air(int width, int height, int target, int gravity, double friction, 
	     double fmax, Console c) { 
    WIDTH = width;
    HEIGHT = height;
    MAVlist = new Vector(100); 
    setBackground( new Color(135, 206, 250) );
    DIST = target;
    G = gravity;
    Fmax = fmax;
    FR = friction;
    main = c;
    setSize( width, height );

    // listener for mouse clicks to determine blast position
    blastListener = new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	removeMouseListener( this );
	setCursor( defaultCursor );
	Point blastCenter = new Point( e.getX(), e.getY() );
	Blast blast = new Blast( blastCenter );
	blast.start();
	for( Enumeration mavs = getMAVs(); mavs.hasMoreElements(); ) {
	  MAV nextMAV = (MAV)mavs.nextElement();
	  nextMAV.blast( blastCenter );
	}
      }
    };
  }

  public void start() {
    setRepulse( true );
    // test test !!!!
    Vx = Vy = 0.0;
    vcount = 0;
  }

  public void stop() {
    for( Enumeration mavs = MAVlist.elements(); mavs.hasMoreElements(); ) {
      ((MAV)mavs.nextElement()).interrupt();
    }
    for( Enumeration mavs = MAVlist.elements(); mavs.hasMoreElements(); ) {
      try { ((MAV)mavs.nextElement()).join(); }
      catch (Exception e) {}
    }
    Graphics g = getGraphics();
    g.setColor( getBackground() );
    g.fillRect( 0, 0, WIDTH, HEIGHT );
    MAVlist.clear();
  }

  // ActionListener interface (*)
  public void actionPerformed( ActionEvent e ) {
    // blast!!!
    System.out.println( "blast" );
    // remember the default cursor for the component
    defaultCursor = getCursor();
    setCursor( bombCursor );
    addMouseListener( blastListener );

//     System.out.println( "# blast!!!!!" );
//     Point2D blastCenter = new Point2D.Double(WIDTH/2-150,HEIGHT/2-150);
//     for( Enumeration mavs = getMAVs(); mavs.hasMoreElements(); ) {
//       MAV nextMAV = (MAV)mavs.nextElement();
//       nextMAV.blast( blastCenter );
//     }
//     Blast blast = new Blast( blastCenter );
//     blast.start();
  }

  public void addMAV( MAV newMAV ) { MAVlist.addElement( newMAV ); }

  public Enumeration getMAVs() { return MAVlist.elements(); }

  public void setBombCursor( Cursor c ) { bombCursor = c; }

  public void setRepulse( boolean r ) { repulse = r; System.out.print("repulsion "); if(r) System.out.println("on"); else System.out.print("off"); }

  public boolean getRepulse() { return repulse; }

  public void useSoftwareEmulation() {
    for( Enumeration mavs = getMAVs(); mavs.hasMoreElements(); ) 
      ((MAV)mavs.nextElement()).useSoftwareEmulation();
  }
  
  // initial placement of an MAV
  public void drawMAV( Point2D newPt ) {
    Graphics g = getGraphics();

    // translate coordinates into integers
    int x = (int)Math.round( newPt.getX() );
    int y = (int)Math.round( newPt.getY() );
    
    g.setColor( MAV_COLOR );
    g.setPaintMode( );
    g.fillOval( x-MAV_RAD, y-MAV_RAD, 2*MAV_RAD, 2*MAV_RAD );
  }

  // moves an MAV to a new position; old is for erasing
  public synchronized void redrawMAV( Point2D oldPt, Point2D newPt ) {
    Graphics g = getGraphics();
    
    // translate coordinates into integers
    int oldx = (int)Math.round( oldPt.getX() );
    int oldy = (int)Math.round( oldPt.getY() );
    int newx = (int)Math.round( newPt.getX() );
    int newy = (int)Math.round( newPt.getY() );
    
    // erase the old position
//    g.setColor( Color.black );
//    g.setXORMode( getBackground() );
    g.setColor( getBackground() );
    g.setPaintMode();
    g.fillOval( oldx-MAV_RAD, oldy-MAV_RAD, 2*MAV_RAD, 2*MAV_RAD );
    // draw the new position
    g.setColor( MAV_COLOR );
    g.setPaintMode( );
    g.fillOval( newx-MAV_RAD, newy-MAV_RAD, 2*MAV_RAD, 2*MAV_RAD );
  }

  public void paint( Graphics g ) { 
    g.setColor( getBackground() );
    g.fillRect( 0, 0, WIDTH, HEIGHT );
    for( Enumeration mavs = MAVlist.elements(); mavs.hasMoreElements(); ) {
	((MAV)mavs.nextElement()).updatePosition();
    }
  }

  // test test !!!!!
  synchronized public void addVector( double x, double y ) {
    Vx += x;  Vy += y;
  }
  synchronized public void compensate() { 
    for( Enumeration mavs = MAVlist.elements(); mavs.hasMoreElements(); ) {
	((MAV)mavs.nextElement()).correction();
    }
    Vx = Vy = 0;
    vcount = 0;
  }
  // internal class to draw a blast
  class Blast extends Thread {
    int x, y;

    Blast( Point2D pos ) { x = (int)pos.getX(); y = (int)pos.getY(); }

    public void run() {
      Graphics g = getGraphics();
      Color gc = g.getColor();
      g.setColor( Color.red );
      for( int i = 1; i <= 40; i = i+2 ) {
	g.drawOval( x-i, y-i, 2*i, 2*i );
	try { sleep( 15 ); } catch( Exception e ) { }
      }
      g.setColor( getBackground() );
      for( int i = 1; i <= 40; i = i+2 ) {
	g.drawOval( x-i, y-i, 2*i, 2*i );
	try { sleep( 15 ); } catch( Exception e ) { }
      }
      g.setColor( gc );
    }
  }; // internal class Blast

}; // class Air
